/***************************************************************************
                      evolot_run.h  -  description
                             -------------------
    begin                : Sun Nov 17 2002
    copyright            : (C) 2002 by Gerhard Jaeger
    email                : GerhardJaeger@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*!\file evolot_run.h
   \brief Header file of the implementation of the actual program of the evolOT project

*/


#ifndef EVOLOTRUN_H_
#define EVOLOTRUN_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "normal_dist.h"
#include "eval.h"

#include <iostream>
#include <ctime>
#include <fstream>
#include <qapplication.h>
#include <qdialog.h>
#include <qprogressdialog.h>
#include <qtable.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qlcdnumber.h>
//#include <vector>
using namespace std;


#define LineLength 500

//! Implementation of the Iterated Bidirectional Gradual Learning Algorithm
/*! The 'run()' procedure *is* the Iterated Bidirectional Gradual Learning Algorithm.
    The public attributes contain parameters of the algorithm.
*/
class Evolotrun
{
 public:
  
  //! Iterated Bidirectional Gradual Learning Algorithm
  void run();
  
 private:
  //! Sets the ranking of all constraints to InitialValue.
  void InitializeGLA(vector<double> &constraintarray);
  
  //! Reads the content of gen_file (a table of integers) into the vector GEN.
  /*! GEN represents a two-dimensional array, with 'Evolotrun::NInputs' rows and 
      'Evolotrun::NOutputs' columns
   */
  void GetGenerator(QString gen_file, vector<int> &GEN);

  //! Reads the content of starfile into the vector 'violates'
  /*! 'starfile' is a text file containing a sequence of tables (representing constraint
      violation patterns). They are stored in the vector 'violates'. It represents a 
      3D-vector (with Evolotrun::NInputs layers, Evolotrun::NOutputs rows, and 
      Evolot::NConstraints columns)
  */
  void GetViolationPattern(QString starfile, vector<int> &violates);

  //! puts next non-empty line from input_file into line, ignoring material starting with '%'
  void fgets_no_comments(char * line, FILE * input_file);

  //! returns random double between 0 and 1
  double RandomNumber(void);

  //! draws a random element from 'generator'
  /*! 'generator' is understood as Evolot::NInputs x Evolot::NOutputs 2D-structure.
      The random candidate is stored  in (row,column) format in (candidate[0],candidate[1])
  */
  void RandomCandidate(int candidate[], vector<int> &generator);

  //! adds noise to constraint rankings
  /*! For each element GrammarValues[i], a random amount (drawn from N(0,Evolot::Noise)) 
      is added, and the result is stored in Evolot::ActualValues[i]
  */
  void AddNoise(vector<double> &GrammarValues);
  
  //! sorts constraints in descending order of their actual values
  /*! the integers i \f$\in\f$ {0 ... Evolot::NConstraints-1} represent the constraints 
      and Evolotrun::ActualValues[i] their ranking on a continuous scale. NewRanking stores
      the numbers of the constraints in descending order of their Actual Values.
   */
  void RandomRanking(vector<double> &GrammarValues, vector<int> &NewRanking);

  //! an implementation of quicksort with (Evolotrun::ActualValues, >) as sorting criterion
  void gjsort(vector<int> &bag);
  
  void quicksort(vector<int> &bag, int p, int size_of_bag);
  int partition(vector<int> &bag, int p, int r);

  //! displays the Evolot::NInputs x Evolot::NOutputs matrix 'vector' in the table 't'
  void display_gen(vector<int> &GEN, QTable *t);
  
  
 public:
  int NInputs;
  int NOutputs;
  int NConstraints;
  int NGenerations;
  int NEquivalenceClasses;
  int NObservations;
  int StepSize;
  double InitialValue;
  double Noise;
  double Plasticity;

  //! contains the frequency distributions of the initial training corpus
  QString genfile;
  
  //! stores the constraint rankings (after iterated / during) learning
  QString con_outputfile;
  
  //! stores the frequency distributions in each generation
  QString gen_outputfile;
  
  //! contains the constraint violation patterns
  QString starfile;

  //! if 'true', optimality is bidirectional, else unidirectional
  bool bi_eval;

  //! if 'true' learning is bidirectional, else unidirectional
  bool bi_learn;
  
  //! if 'true' learning is iterated
  /*! in this case the result of learning after Evolotrun::NObservations is sent to
      Evolotrun::con_outputfile, the constraints are reset to Evolotrun::InitialValue, 
      and the procedure is repeated Evolotrun::NGenerations times. If 'evolution' is false, 
      only one learning process is simulated, and always after Evolotrun::StepSize many 
      observations the intermediate result is sent to the output.
  */
  bool evolution;
  
  QTable *table;
  QLCDNumber *lcd;
  
  
 private:
  //! contains the actual values of the constraints (GrammarValue + Noise)
  vector<double> ActualValues;
  
}
;

#endif
