/***************************************************************************
                      evolotimpl.h  -  description
                             -------------------
    begin                : Sun Nov 17 2002
    copyright            : (C) 2002 by Gerhard Jaeger
    email                : GerhardJaeger@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*!\file evolotimpl.h 
  \brief Header of interface between GUI and actual program of evolOT

   Contains the header of the class 'Evolot', a sub-class of the GUI class
   'evolotBase'. This class stores the information from the GUI in its members and 
   passes them on to 'Evolotrun' and 'StarmakerRun'.  
*/

#ifndef EVOLOTIMPL_H
#define EVOLOTIMPL_H

#include "evolot.h"
#include "evolot_run.h"
#include "starmakerrun.h"
#include <qspinbox.h>
#include <qfiledialog.h>
#include <qpushbutton.h>
#include <qsize.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmap.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qcheckbox.h>

//! implements the slots from the GUI
/*! and passes the information from the GUI to parameters that can be used
  by Evolotrun::run() and StarmakerRun::run()
*/
class Evolot : public evolotBase
{
  Q_OBJECT
    
    public:
  //! class constructor
  /*! sets some default values and initializes the table and lcd number
   */
  Evolot( QWidget* parent = 0, const char* name = 0,
	  bool modal = FALSE, WFlags f = 0 );
  

  //! class destructor
  /*! removes the table and the lcd number
   */
  ~Evolot(void);
  
  public slots:
    //! evolution or learning?
    virtual void evolution_slot(const QString&);

  //! bidirectional or unidirectional evaluation?
  virtual void bieval_slot(bool);
  
  //! bidirectional or unidirectional learning?
  virtual void bilearn_slot(bool);
  
  //! file to store frequencies?
  virtual void freqfileDialogslot();
  
  //! file to store frequencies?
  virtual void freqfile_slot(const QString&);
  
  //! file containing description of constraints?
  virtual void scriptfileDialogslot();

  //! file containing description of constraints?
  virtual void scriptfile_slot(const QString&);

  //! file containing constraint violation patterns?
  virtual void starfileDialogslot();

  //! file containing constraint violation patterns?
  virtual void starfile_slot(const QString&);
  
  //! file containing initial training corpus?
  virtual void genfileDialogslot();

  //! file containing initial training corpus?
  virtual void genfile_slot(const QString&);

  //! output file for constraint rankings?
  virtual void rankingfileDialogslot();

  //! output file for constraint rankings?
  virtual void rankingfile_slot(const QString&);

  //! initial constraint ranking?
  virtual void initial_slot(int);
  
  //! number of constraints?
  virtual void nconstraints_slot(int);
  
  //! number of equivalence classes?
  virtual void neq_slot(int);

  //! number of generations?
  virtual void ngenerations_slot(int);

  //! number of inputs?
  virtual void ninput_slot(int);

  //! number of obserations?
  virtual void nobservations_slot(int);

  //! standard deviation of noise?
  virtual void noise_slot(int);
  
  //! number of outputs?
  virtual void noutput_slot(int);

  //! plasiticity?
  virtual void plasticity_slot(int);

  //! stepsize?
  /*! number of observations between two snapshots if one learning process is recorded
   */
  virtual void stepsize_slot(int);
  
  virtual void compile_slot();

  virtual void run_slot();

 public:
  int NInputs;
  int NOutputs;
  int NConstraints;
  int NGenerations;
  int NEquivalenceClasses;
  int NObservations;
  int StepSize;
  double InitialValue;
  double Noise;
  double Plasticity;
  QTable *table;
  QLCDNumber *lcd;
  
  //! contains the frequency distributions of the initial training corpus
  QString genfile;

  //! stores the constraint rankings (after iterated / during) learning
  QString con_outputfile;

  //! stores the frequency distributions in each generation
  QString gen_outputfile;

  //! contains the description of the constraint violation patterns
  QString scriptfile;

  //! contains the constraint violation patterns
  QString starfile;

  //! if 'true' learning is iterated
  /*! in this case the result of learning after Evolotrun::NObservations is sent to
      Evolotrun::con_outputfile, the constraints are reset to Evolotrun::InitialValue, 
      and the procedure is repeated Evolotrun::NGenerations times. If 'evolution' is false, 
      only one learning process is simulated, and always after Evolotrun::StepSize many 
      observations the intermediate result is sent to the output.
  */
  bool evolution;

  //! if 'true', optimality is bidirectional, else unidirectional
  bool bi_eval;

  //! if 'true' learning is bidirectional, else unidirectional
  bool bi_learn;
};



#endif
